@@ -17,7 +17,7 @@
 # File Name : sos-gov.py
 # Creation Date : 10-01-2013
 # Created By : Jamie Duncan
-# Last Modified : Sun 20 Jul 2014 10:51:25 AM EDT
+# Last Modified : Sat 13 Sep 2014 10:51:54 PM EDT
 # Purpose : an sosreport scrubber
 
 import os
@@ -42,34 +42,50 @@ class SOSCleaner:
     def __init__(self, quiet=False):
 
         self.name = 'soscleaner'
-        self.version = '0.2.1'
+        self.version = '0.2.2'
         self.loglevel = 'INFO' #this can be overridden by the command-line app
         self.quiet = quiet
         self.domain_count = 0
         self.domains = list()
+        self.keywords = list()
         self.domainname = None
+        self.report_dir = '/tmp'
 
-        #IP obfuscation information
+        # IP obfuscation information
         self.ip_db = dict() #IP database
         self.start_ip = '10.230.230.1'
 
-        #Hostname obfuscation information
+        # Hostname obfuscation information
         self.hn_db = dict() #hostname database
         self.hostname_count = 0
         self.hostname = None
 
-        #Domainname obfuscation information
+        # Domainname obfuscation information
         self.dn_db = dict() #domainname database
         self.root_domain = 'example.com' #right now this needs to be a 2nd level domain, like foo.com, example.com, domain.org, etc.
-        self.origin_path, self.dir_path, self.session, self.logfile, self.uuid = self._prep_environment()
-        self._start_logging(self.logfile)
+
+        # self.origin_path, self.dir_path, self.session, self.logfile, self.uuid = self._prep_environment()
+        # self._start_logging(self.logfile)
+
+        # Keyword obfuscation information
+        self.keywords = None
+        self.kw_db = dict() #keyword database
+        self.kw_count = 0
 
         self.magic = magic.open(magic.MAGIC_NONE)
         self.magic.load()
 
     def _check_uid(self): # pragma no cover
-        if os.getuid() != 0:
-            raise Exception("You Must Execute soscleaner As Root")
+
+        try:
+            if os.getuid() != 0:
+                self.logger.con_out("soscleaner must be executed by the root user in the same manner as sosreport")
+                self.logger.con_out("soscleaner cannot continue. Exiting...")
+
+                sys.exit(8)
+
+        except Exception, e:    # pragma: no cover
+            self.logger.exception(e)
 
     def _skip_file(self, d, files):
         '''
@@ -127,11 +143,11 @@ class SOSCleaner:
     def _prep_environment(self):
 
         #we set up our various needed directory structures, etc.
-        ran_uuid = str(uuid.uuid4().int)[:16]                       # 16 digit random string
-        origin_path = "/tmp/soscleaner-origin-%s" % ran_uuid        # the origin dir we'll copy the files into
-        dir_path = "/tmp/soscleaner-%s" % ran_uuid                  # the dir we will put our cleaned files into
-        session = "soscleaner-%s" % ran_uuid                        # short-hand for the soscleaner session to create reports, etc.
-        logfile = "/tmp/%s.log" % session                       # the primary logfile
+        ran_uuid = str(uuid.uuid4().int)[:16]                                                 # 16 digit random string
+        origin_path = os.path.join(self.report_dir, "soscleaner-origin-%s" % ran_uuid)        # the origin dir we'll copy the files into
+        dir_path = os.path.join(self.report_dir, "soscleaner-%s" % ran_uuid)                  # the dir we will put our cleaned files into
+        session = os.path.join(self.report_dir, "soscleaner-%s" % ran_uuid)                   # short-hand for the soscleaner session to create reports, etc.
+        logfile = os.path.join(self.report_dir, "%s.log" % session)                           # the primary logfile
 
         return origin_path, dir_path, session, logfile, ran_uuid
 
@@ -208,7 +224,7 @@ class SOSCleaner:
         this will take the obfuscated ip and hostname databases and output csv files
         '''
         try:
-            ip_report_name = "/tmp/%s-ip.csv" % self.session
+            ip_report_name = os.path.join(self.report_dir, "%s-ip.csv" % self.session)
             self.logger.con_out('Creating IP Report - %s', ip_report_name)
             ip_report = open(ip_report_name, 'w')
             ip_report.write('Obfuscated IP,Original IP\n')
@@ -224,7 +240,7 @@ class SOSCleaner:
 
     def _create_hn_report(self):
         try:
-            hn_report_name = "/tmp/%s-hostname.csv" % self.session
+            hn_report_name = os.path.join(self.report_dir, "%s-hostname.csv" % self.session)
             self.logger.con_out('Creating Hostname Report - %s', hn_report_name)
             hn_report = open(hn_report_name, 'w')
             hn_report.write('Obfuscated Hostname,Original Hostname\n')
@@ -243,7 +259,7 @@ class SOSCleaner:
 
     def _create_dn_report(self):
         try:
-            dn_report_name = "/tmp/%s-dn.csv" % self.session
+            dn_report_name = os.path.join(self.report_dir, "%s-dn.csv" % self.session)
             self.logger.con_out('Creating Domainname Report - %s', dn_report_name)
             dn_report = open(dn_report_name, 'w')
             dn_report.write('Obfuscated Domain,Original Domain\n')
@@ -292,7 +308,7 @@ class SOSCleaner:
 
     def _make_dest_env(self):
         '''
-        This will create the folder in /tmp to store the sanitized files and populate it using shutil
+        This will create the folder in self.report_dir (defaults to /tmp) to store the sanitized files and populate it using shutil
         These are the files that will be scrubbed
         '''
         try:
@@ -305,13 +321,13 @@ class SOSCleaner:
     def _create_archive(self):
         '''This will create a tar.gz compressed archive of the scrubbed directory'''
         try:
-            self.archive_path = "/tmp/%s.tar.gz" % self.session
+            self.archive_path = os.path.join(self.report_dir, "%s.tar.gz" % self.session)
             self.logger.con_out('Creating SOSCleaner Archive - %s', self.archive_path)
             t = tarfile.open(self.archive_path, 'w:gz')
             for dirpath, dirnames, filenames in os.walk(self.dir_path):
                 for f in filenames:
                     f_full = os.path.join(dirpath, f)
-                    f_archive = f_full.replace('/tmp','')
+                    f_archive = f_full.replace(self.report_dir,'')
                     self.logger.debug('adding %s to %s archive', f_archive, self.archive_path)
                     t.add(f_full, arcname=f_archive)
         except Exception,e: #pragma: no cover
@@ -322,7 +338,7 @@ class SOSCleaner:
         self.logger.info('Archiving Complete')
         self.logger.con_out('SOSCleaner Complete')
         if not self.quiet:  # pragma: no cover
-            t.add(self.logfile, arcname=self.logfile.replace('/tmp',''))
+            t.add(self.logfile, arcname=self.logfile.replace(self.report_dir,''))
         t.close()
 
     def _clean_up(self):
@@ -338,6 +354,31 @@ class SOSCleaner:
         except Exception, e:    #pragma: no cover
             self.logger.exception(e)
 
+    def _process_hosts_file(self):
+        # this will process the hosts file more thoroughly to try and capture as many server short names/aliases as possible
+        # could lead to false positives if people use dumb things for server aliases, like 'file' or 'server' or other common terms
+        # this may be an option that can be enabled... --hosts or similar?
+
+        try:
+            if os.path.isfile(os.path.join(self.dir_path, 'etc/hosts')):
+                with open(os.path.join(self.dir_path, 'etc/hosts')) as f:
+                    self.logger.con_out("Processing hosts file for better obfuscation coverage")
+                    data = f.readlines()
+                    for line in data:
+                        x = re.split('\ |\t', line.rstrip())    #chunk up the line, delimiting with spaces and tabs (both used in hosts files)
+                        # we run through the rest of the items in a given line, ignoring the IP to be picked up by the normal methods
+                        # skipping over the 'localhost' and 'localdomain' entries
+                        for item in x[1:len(x)]:
+                            if len(item) > 0:
+                                if all(['localhost' not in item, 'localdomain' not in item]):
+                                    new_host = self._hn2db(item)
+                                    self.logger.debug("Added to hostname database through hosts file processing - %s > %s", item, new_host)
+            else: # pragma: no cover
+                self.logger.con_out("Unable to Process Hosts File. Hosts File Processing Disabled")
+
+        except Exception, e:    #pragma: no cover
+            self.logger.exception(e)
+
     def _domains2db(self):
         #adds any additional domainnames to the domain database to be searched for
         try:
@@ -361,6 +402,49 @@ class SOSCleaner:
         except Exception, e: # pragma: no cover
             self.logger.exception(e)
 
+    def _keywords2db(self):
+        #processes optional keywords to add to be obfuscated
+        try:
+            if self.keywords:   # value is set to None by default
+                k_count = 0
+                for f in self.keywords:
+                    if os.path.isfile(f):
+                        with open(f, 'r') as klist:
+                            for keyword in klist.readlines():
+                                o_kw = "keyword%s" % k_count
+                                self.kw_db[keyword.rstrip()] = o_kw
+                                self.logger.con_out("Added Obfuscated Keyword - %s", o_kw)
+                                k_count += 1
+                        self.logger.con_out("Added Keyword Contents from file - %s", f)
+
+                    else:
+                        self.logger.con_out("%s does not seem to be a file. Not adding any keywords from" % f)
+
+            self.kw_count = k_count
+
+        except Exception, e: # pragma: no cover
+            self.logger.exception(e)
+
+    def _kw2db(self, keyword):
+        #returns the obfuscated value for a keyword
+
+        return self.kw_db[keyword]
+
+    def _sub_keywords(self, line):
+        # this will substitute out any keyword entries on a given line
+        #try:
+        if self.kw_count > 0:    # we have obfuscated keywords to work with
+            for k in self.kw_db.keys():
+                if k in line:
+                    line = line.replace(k, self._kw2db(k))
+                    self.logger.debug("Obfuscating Keyword - %s > %s", k, self._kw2db(k))
+
+        return line
+
+        '''except Exception, e: # pragma: no cover
+            self.logger.exception(e)
+            raise Exception('SubKeywordError: Unable to Substitute Keywords')'''
+
     def _get_hostname(self):
         #gets the hostname and stores hostname/domainname so they can be filtered out later
 
@@ -484,8 +568,9 @@ class SOSCleaner:
     def _clean_line(self, l):
         '''this will return a line with obfuscations for all possible variables, hostname, ip, etc.'''
 
-        new_line = self._sub_ip(l)  #IP substitution
-        new_line = self._sub_hostname(new_line)    #Hostname substitution
+        new_line = self._sub_ip(l)                  # IP substitution
+        new_line = self._sub_hostname(new_line)     # Hostname substitution
+        new_line = self._sub_keywords(new_line)     # Keyword Substitution
 
         return new_line
 
@@ -550,7 +635,7 @@ class SOSCleaner:
                 os.makedirs(self.dir_path)    # create the dir_path directory
             self._add_extra_files(files)
 
-        except OSError, e:
+        except OSError, e: # pragma: no cover
             if e.errno == errno.EEXIST:
                 pass
             else:   # pragma: no cover
@@ -564,11 +649,18 @@ class SOSCleaner:
     def clean_report(self, options, sosreport): # pragma: no cover
         '''this is the primary function, to put everything together and analyze an sosreport'''
 
+        if options.report_dir: # override the default location for artifacts (/tmp)
+            if os.path.isdir(options.report_dir):
+                self.report_dir = options.report_dir
+        self.origin_path, self.dir_path, self.session, self.logfile, self.uuid = self._prep_environment()
+        self._start_logging(self.logfile)
         self._check_uid() #make sure it's soscleaner is running as root
         self._get_disclaimer()
         if options.domains:
             self.domains = options.domains
-
+        if options.keywords:
+            self.keywords = options.keywords
+            self._keywords2db()
         if not sosreport:
             if not options.files:
                 raise Exception("Error: You must supply either an sosreport and/or files to process")
@@ -576,16 +668,18 @@ class SOSCleaner:
             self.logger.con_out("No sosreport supplied. Only processing specific files")
             self._clean_files_only(options.files)
 
-        else:   #we DO have an sosreport to analyze
+        else:   # we DO have an sosreport to analyze
             self.report = self._extract_sosreport(sosreport)
-            self._make_dest_env()   #create the working directory
+            self._make_dest_env()   # create the working directory
             self.hostname, self.domainname = self._get_hostname()
 
             if options.files:
                 self._add_extra_files(options.files)
 
-            if self.hostname:   #if we have a hostname that's not a None type
-                self.hn_db['host0'] = self.hostname     #we'll prime the hostname pump to clear out a ton of useless logic later
+            if self.hostname:   # if we have a hostname that's not a None type
+                self.hn_db['host0'] = self.hostname     # we'll prime the hostname pump to clear out a ton of useless logic later
+
+            self._process_hosts_file()  # we'll take a dig through the hosts file and make sure it is as scrubbed as possible
 
         self._domains2db()
         files = self._file_list(self.dir_path)
